;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x2fcf74f1641172eaa6c3e6d6e0dab4d0 . "./code.cg2js/cg-acache.fasl")
(#x2304e4b02df392c180927779a815c2c5 . "./code.cg2js/cg-base.fasl")
(#x16c2442c9c6c95cc320eb6b085b01d3e . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x0b56e327a6a92c65050d1e3c7e587621 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x737f675e2d4b10b6f3936fbe69506c85 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x6b53e35a88b1961cc99f06911ee8edfe . "./code.cg2js/cg-button.fasl")
(#x6d528533a1d1762f764c99d5d6418aff . "./code.cg2js/cg-calendar.fasl")
(#x2da8c51f7caf46c13611d349fc81ac38 . "./code.cg2js/cg-caret.fasl")
(#xe76734c10fbe4995c469e7c7fdf5494c . "./code.cg2js/cg-chart-or-plot.fasl")
(#xa98c04aaec8225acb0e3dbe137e05d03 . "./code.cg2js/cg-chart-widget.fasl")
(#x635b58654d0eeecad028ccf579f7b8f6 . "./code.cg2js/cg-check-box.fasl")
(#xb24d0e8658510bfbe0b283fd9266adf5 . "./code.cg2js/cg-choice-list.fasl")
(#x0d6a86852643dd40714f045776b5f049 . "./code.cg2js/cg-choose-printer.fasl")
(#xe74c88e2062d4531ee8f87e373c5e68c . "./code.cg2js/cg-class-grid.fasl")
(#x7d0e890c1ca7bfde53c634c46b2e10da . "./code.cg2js/cg-class-slot-grid.fasl")
(#xc2cd5b62f94b308baf96c7ef840cff88 . "./code.cg2js/cg-class-support.fasl")
(#x917f5c52b22134d1905798f7e8aa8f36 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xdceb587ba54aa668395b0d808e919769 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x457765af2796abc7247dcbd02457f8e2 . "./code.cg2js/cg-clipboard.fasl")
(#x64ae721412ae2421d0b117e9a7029e34 . "./code.cg2js/cg-color-dialog.fasl")
(#x229cea4af01e0611cd11285c39c40600 . "./code.cg2js/cg-combo-box.fasl")
(#x190977094e848c80e9e5865612290a48 . "./code.cg2js/cg-common-control.fasl")
(#x417c491956b74cf4be7d9306ff6afc7b . "./code.cg2js/cg-comtab.fasl")
(#xe33ebb28e93d65e3008c645f1ed42ce0 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xe5090cd4b86dfb4d9e10d8bde4b23a38 . "./code.cg2js/cg-curve.fasl")
(#x30fe60efb3ce9da666b8b4cc699eded2 . "./code.cg2js/cg-dde-utils.fasl")
(#x43e3460037b97a7baffa0241ac82cf88 . "./code.cg2js/cg-dev.fasl")
(#x13bae5d3e0135b5af115bdfc6baefcc8 . "./code.cg2js/cg-dialog-item.fasl")
(#x3d4d846f16b48a45015d34b4fe0c0b6f . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xb52fb3f4bcda030e2e2efaa310de4c10 . "./code.cg2js/cg-directory-dialog.fasl")
(#xc958cfdf06db8fdc9b2c6e77c79f7929 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xf209286a6d909f7bde27b7e4a9af9150 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x10479bd290302198df550c42086cfbfb . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x9684f8c576ed30d088ef0a623056363c . "./code.cg2js/cg-drawable.fasl")
(#x4a8704809d76efd6e9717c1e7cd55d08 . "./code.cg2js/cg-dropping-outline.fasl")
(#xdae360ecbe07dac97b04c43a74709908 . "./code.cg2js/cg-edit-in-place.fasl")
(#xfa885d1d72bde3d95857cb07bf0015df . "./code.cg2js/cg-editable-text.fasl")
(#x65d03918989e9316a3027fa16b7125bd . "./code.cg2js/cg-file-dialog.fasl")
(#x5713ae8a02e62cfb8a0b4a45c6a540c2 . "./code.cg2js/cg-fill-texture.fasl")
(#x04d78741fae3d8cc0ff00b0f2c2c2cf5 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xb59349de794345231c1c82c96326dc72 . "./code.cg2js/cg-font-dialog.fasl")
(#xcbea14e7f3249bd8cefdcbfc7f1a4e25 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x952a48fe29ffc687f1deece9b5fe78f2 . "./code.cg2js/cg-get-pixmap.fasl")
(#xc7558de813f0bbde32fc0fbfa79b2434 . "./code.cg2js/cg-get-position.fasl")
(#x7dd4236f2bc0bb2a8d0be5b9b3abaeca . "./code.cg2js/cg-graphics-context.fasl")
(#x312c682832370bbffa456fecd0abb97c . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x53f1e583ddd4e6d1e1a1ed4dd6cfb276 . "./code.cg2js/cg-grid-widget.fasl")
(#x996e1b8739464e21b95ed52fa687943f . "./code.cg2js/cg-group-box.fasl")
(#x4274309d152bd1d085361578fc7f63b9 . "./code.cg2js/cg-header-control.fasl")
(#x3a8b5b09f11703d2d5d2a66b8bb09050 . "./code.cg2js/cg-hotspot.fasl")
(#xe9d5b10079e17f3d90056d36bddaeef0 . "./code.cg2js/cg-html-dialog.fasl")
(#x5d60de017d3376d0a2e24485c856f5ce . "./code.cg2js/cg-html-widget.fasl")
(#x6e857ee71e8a177e8e3f2399aac49558 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x4147ac4f591eaf063af3c7d9e1320a56 . "./code.cg2js/cg-icon.fasl")
(#x07785e432320fff7ec5d18f51cca235d . "./code.cg2js/cg-ie.fasl")
(#x9bef171b8bb3e364d8f2ea083173eb6b . "./code.cg2js/cg-intersect-posbox.fasl")
(#xc4c37f27bae5e3497a13230db4f58393 . "./code.cg2js/cg-intersect.fasl")
(#xc196f359ff10b8fb6ac220d70ba16fe5 . "./code.cg2js/cg-item-list.fasl")
(#x8ab08fc55a8652db11e954bda93589cd . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xff0ad2e286f9406f86e0fdb9459c96e9 . "./code.cg2js/cg-lamp.fasl")
(#x31998432251d55251317b7d3069b571a . "./code.cg2js/cg-layout.fasl")
(#x7d43d7ad99b7ccd32ca897553bc3016d . "./code.cg2js/cg-lettered-menu.fasl")
(#xf5447dc4aa951f152cf92ddc66e92ef1 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xe1ca917f4866d3be760334da5ab79f6f . "./code.cg2js/cg-lisp-text.fasl")
(#x3325178fb41aaaaac1383df45a0d98aa . "./code.cg2js/cg-lisp-widget.fasl")
(#x7bc6b5656710861c763d7228913e44de . "./code.cg2js/cg-list-view.fasl")
(#xcd1ff6c0e49c21c2e9cd4721fdb26ddf . "./code.cg2js/cg-mci.fasl")
(#xf7ffd5d8eb6fa8a56f26624716723047 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x1272c554cca11a20773b8cc1f9d5cdc2 . "./code.cg2js/cg-menu.fasl")
(#xc570a904ce367b1b684feae2d6acc499 . "./code.cg2js/cg-message-dialog.fasl")
(#x7d4a1458e93051e05dd9fc3f95458e1f . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x21ab894356eec66ad92a99b9d0277983 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x17ad5aee5e0f01b874f4cb0676314df1 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xb8aee4869a0f419bc09b76adda7b3efb . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x4de91af6f8c069ada2676652e6bfe701 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x7c5c90d61cb75996ad64b3926a73bf42 . "./code.cg2js/cg-nodes.fasl")
(#x002c218c4692dae5af64b2c559227363 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xc4f3f532fcda6e220a1806ea54cf652d . "./code.cg2js/cg-object-editor.fasl")
(#x9b3b2cdde4b1c9cfbbb8d8bb508fd61f . "./code.cg2js/cg-ocx.fasl")
(#x5547f44a6f440ca71f88a8a12c66ac07 . "./code.cg2js/cg-os-widget.fasl")
(#xc6d536cc75c2857cf45264eaf8d126ae . "./code.cg2js/cg-os-window.fasl")
(#x180b30f73864a73ef5eda724e655d49e . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x73f38f889b4e2bc12ad4ad3b6cc638a7 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x8b9485f77101865902dc57a1e2d64efd . "./code.cg2js/cg-outline.fasl")
(#x201b430ae48c26df609ad2dc3f430d34 . "./code.cg2js/cg-palette.fasl")
(#xdf122c4def9337f22781938932e026f3 . "./code.cg2js/cg-paren-matching.fasl")
(#x931f99fe9c999478f6598c6580c12d1a . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x0ba6156a4648ee75471d98838b7ea998 . "./code.cg2js/cg-picture-widget.fasl")
(#x62820c6347d1dbc21b253ef5b45c244a . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xecf593da863a887f34600b13fc055f87 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xddeb7c8a02641a8305eccab773da413e . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xf6ac6de9f819658d758a17ef3c696d1f . "./code.cg2js/cg-pixmap-widget.fasl")
(#x5e48f60e8a63c92c01470bde0de06454 . "./code.cg2js/cg-pixmap.fasl")
(#x6ec80fd60a900d9664726a72129bf570 . "./code.cg2js/cg-player.fasl")
(#x7971be7c1feea23260051513077e37f4 . "./code.cg2js/cg-plot-widget.fasl")
(#x37c819a2e8f001ad1888cc7c5d3db93f . "./code.cg2js/cg-printing.fasl")
(#x452117a9342e0797873f58777d6c932d . "./code.cg2js/cg-progress-indicator.fasl")
(#x42eb7b80d427f84a43f9dc4df25f1b3d . "./code.cg2js/cg-project-window.fasl")
(#x44e1381bba0e1eefa2fb9e7089e01274 . "./code.cg2js/cg-property.fasl")
(#x4a48b8d859927ad3972e6bf0fa212e91 . "./code.cg2js/cg-radio-button.fasl")
(#x1fa268282fe4ec962effa845fe44badd . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x44c359f43cb8d3e50a533b5dc61c655d . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xdc6c4c1b1ef89c3efcf2617b0272a501 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xe01034f9534550d77c0dd7e5cc6b375f . "./code.cg2js/cg-rich-edit.fasl")
(#xfe914ef5e76cb181134d8e783359960b . "./code.cg2js/cg-sample-file-menu.fasl")
(#x9b6d5a1a32bf406c3e91393c8b3f27b5 . "./code.cg2js/cg-scaling-stream.fasl")
(#x0e0011103ab5ca2eaa90be8bd187506d . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x8f3c06a319ed61c16b6453984adf208c . "./code.cg2js/cg-scroll-bar.fasl")
(#x507ba48968c4ebaeef7eea71ae21f352 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x2e082639ad001069de7cfdde43182002 . "./code.cg2js/cg-selected-object.fasl")
(#x63750f9c6b7d3a477d67c0766459f5de . "./code.cg2js/cg-shortcut-menu.fasl")
(#xb071a2cd8c8fd57579e78f6518aeed8f . "./code.cg2js/cg-split-bar.fasl")
(#xf080b1fe820ba621950f4539d30b3b9d . "./code.cg2js/cg-static-text.fasl")
(#x9b0b5c8f4bde274ec3e4ea4d2f853712 . "./code.cg2js/cg-status-bar.fasl")
(#x182e4893d238a67ab464c26db7287876 . "./code.cg2js/cg-string-dialog.fasl")
(#x2b1a0fc09cfbf5bd66d167146fea3286 . "./code.cg2js/cg-tab-control.fasl")
(#x68be9b2c68eee30f1226876130fd5197 . "./code.cg2js/cg-template-string.fasl")
(#x2b4af592df0f10929c7ab5b2b027408a . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xc050c4bdda459dfa8df07251aad43e3e . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x3e87cd3967a8c27f89f7dca6e532cf76 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x6a8ba7cd57cafebf97c865d560d6e58e . "./code.cg2js/cg-text-or-combo.fasl")
(#x81bbcf1c54086347c2270e1336e06b03 . "./code.cg2js/cg-text-widget.fasl")
(#x51f43ebd5122b94b8b511186a5cfe3ef . "./code.cg2js/cg-timer.fasl")
(#xc68541034fa26c126b02db41c39e08c5 . "./code.cg2js/cg-toggling-widget.fasl")
(#x29cc658e7bc9171b24a945281ddac604 . "./code.cg2js/cg-toolbar.fasl")
(#xc6f001376d5c1bc7a98c5042735a8632 . "./code.cg2js/cg-tooltip.fasl")
(#xb04acfb70475cedfe30b0b488a96731a . "./code.cg2js/cg-touch.fasl")
(#x2f8c128dcbde1b4f7429833b4eb3ae91 . "./code.cg2js/cg-trackbar.fasl")
(#x209e80eabe15f69e147345ec69ff1c76 . "./code.cg2js/cg-tray.fasl")
(#xc90b7300c8a90dc79a7a04dfdf6c4878 . "./code.cg2js/cg-up-down-control.fasl")
(#xa369a9056630f7cd37ef8ea2f5087087 . "./code.cg2js/cg-utility-dialog.fasl")
(#xe508bf3916f3fb916d2f9860c889d2af . "./code.cg2js/cg-web-browser-dde.fasl")
(#x8cdc5349080104f28007208eef916bbe . "./code.cg2js/cg-web-browser.fasl")
(#xbb38398c3abdc45ea21759e851465c96 . "./code.cg2js/cg-wrap-string.fasl")
(#xb9ff569f9e17250f4c400cdf0c6163fe . "./code.cg2js/cg-yes-no-list.fasl")
(#x0587152f7a7339bbf2e341f7d12bc0e1 . "./code.cg2js/cg.fasl")
(#x7b1f44c4c1f342fbd199a4e16b155930 . "./code.cg2js/cg-yes-no-string.fasl")
(#x9de86ab61955e603a51ace10beb7ad5f . "./code.idejs/ide-acache.fasl")
(#x8918b4658650a89e74b1d1ba056fa51e . "./code.idejs/ide-apropos.fasl")
(#xb80e680dfb46bdb132b07db01b33b630 . "./code.idejs/ide-base.fasl")
(#x9ba3f38497e0d0fc684998484a678b6f . "./code.idejs/ide-class-browser.fasl")
(#xa21f6e1132ef7c53719c1e1b0781b985 . "./code.idejs/ide-class-grapher.fasl")
(#xc9a3a2cbe16668211028c7ad375029f6 . "./code.idejs/ide-clipboard.fasl")
(#x97fd99824659023dd2915bc3622bf6eb . "./code.idejs/ide-dev.fasl")
(#x08e7e2e8882d59d2d91ddda6269df6e7 . "./code.idejs/ide-editor.fasl")
(#x4f0d92f9b0ecd183efb38ab00b11220d . "./code.idejs/ide-find-definitions.fasl")
(#x98b77849e7836caec046256d3fce6244 . "./code.idejs/ide-find-in-files.fasl")
(#xa4797475d023e229fc7be2ec5fd57732 . "./code.idejs/ide-form.fasl")
(#x54c55c6da8e1bb6966fe2c5ba15836d0 . "./code.idejs/ide-grapher.fasl")
(#xf3a9a6966cf6ae6392ac0530ffb94fa1 . "./code.idejs/ide-inspector.fasl")
(#xf1696c5afd4f529a948b99000aab08d2 . "./code.idejs/ide-intro.fasl")
(#xc3536538cac17a18e2803695a4eedb4c . "./code.idejs/ide-keyshort.fasl")
(#xf88a5a3281f7992d92fa786c1e449351 . "./code.idejs/ide-navigator.fasl")
(#x1d63615e80e47368732f325ceb0dbc53 . "./code.idejs/ide-package-list.fasl")
(#x0098f5c83cb9bed70af48c014db85b2b . "./code.idejs/ide-patch-dialog.fasl")
(#x8a13e939fef42c21b4c7de4b92a165c9 . "./code.idejs/ide-process-browser.fasl")
(#x38346138f07d7a6d37f7ce96c4a28f49 . "./code.idejs/ide-project.fasl")
(#xaece85de78872fa106e2d83793d3d24a . "./code.idejs/ide-trace.fasl")
(#xbb45b9050e81dba44b5f2f1cf4101009 . "./code.idejs/ide-window-list.fasl")
(#x8abacd094f62d9672fde85d0ad0d1aad . "./code.idejs/ide.fasl")
(#xde049c9404b4ba293f0cbc6a4817dc96 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
